#pragma once
#include "taihe/common.hpp"
#include "ohos.book.store.CppOrRustBook.proj.0.hpp"
#include "ohos.book.store.CppOrRustBook.abi.1.h"
#include "ohos.book.store.RustBook.proj.1.hpp"
#include "ohos.book.store.CppBook.proj.1.hpp"
namespace ohos::book::store {
struct CppOrRustBook {
    enum class tag_t : int {
        rust,
        cpp,
    };
    union storage_t {
        storage_t() {}
        ~storage_t() {}
        ::ohos::book::store::RustBook rust;
        ::ohos::book::store::CppBook cpp;
    };
    CppOrRustBook(CppOrRustBook const& other) : m_tag(other.m_tag) {
        switch (m_tag) {
        case tag_t::rust:
            new (&m_data.rust) decltype(m_data.rust)(other.m_data.rust);
            break;
        case tag_t::cpp:
            new (&m_data.cpp) decltype(m_data.cpp)(other.m_data.cpp);
            break;
        default:
            break;
        }
    }
    CppOrRustBook(CppOrRustBook&& other) : m_tag(other.m_tag) {
        switch (m_tag) {
        case tag_t::rust:
            new (&m_data.rust) decltype(m_data.rust)(::std::move(other.m_data.rust));
            break;
        case tag_t::cpp:
            new (&m_data.cpp) decltype(m_data.cpp)(::std::move(other.m_data.cpp));
            break;
        default:
            break;
        }
    }
    CppOrRustBook& operator=(CppOrRustBook const& other) {
        if (this != &other) {
            ::std::destroy_at(this);
            new (this) CppOrRustBook(other);
        }
        return *this;
    }
    CppOrRustBook& operator=(CppOrRustBook&& other) {
        if (this != &other) {
            ::std::destroy_at(this);
            new (this) CppOrRustBook(::std::move(other));
        }
        return *this;
    }
    ~CppOrRustBook() {
        switch (m_tag) {
        case tag_t::rust:
            ::std::destroy_at(&m_data.rust);
            break;
        case tag_t::cpp:
            ::std::destroy_at(&m_data.cpp);
            break;
        default:
            break;
        }
    }
    template<typename... Args>
    CppOrRustBook(::taihe::static_tag_t<tag_t::rust>, Args&&... args) : m_tag(tag_t::rust) {
        new (&m_data.rust) decltype(m_data.rust)(::std::forward<Args>(args)...);
    }
    template<typename... Args>
    CppOrRustBook(::taihe::static_tag_t<tag_t::cpp>, Args&&... args) : m_tag(tag_t::cpp) {
        new (&m_data.cpp) decltype(m_data.cpp)(::std::forward<Args>(args)...);
    }
    template<tag_t tag, typename... Args>
    static CppOrRustBook make(Args&&... args) {
        return CppOrRustBook(::taihe::static_tag<tag>, ::std::forward<Args>(args)...);
    }
    template<tag_t tag, typename... Args>
    CppOrRustBook const& emplace(Args&&... args) {
        ::std::destroy_at(this);
        new (this) CppOrRustBook(::taihe::static_tag<tag>, ::std::forward<Args>(args)...);
        return *this;
    }
    template<tag_t tag>
    auto& get_ref() {
        if constexpr (tag == tag_t::rust) {
            return m_data.rust;
        }
        if constexpr (tag == tag_t::cpp) {
            return m_data.cpp;
        }
    }
    template<tag_t tag>
    auto* get_ptr() {
        return m_tag == tag ? &get_ref<tag>() : nullptr;
    }
    template<tag_t tag>
    auto const& get_ref() const {
        if constexpr (tag == tag_t::rust) {
            return m_data.rust;
        }
        if constexpr (tag == tag_t::cpp) {
            return m_data.cpp;
        }
    }
    template<tag_t tag>
    auto const* get_ptr() const {
        return m_tag == tag ? &get_ref<tag>() : nullptr;
    }
    template<tag_t tag>
    bool holds() const {
        return m_tag == tag;
    }
    tag_t get_tag() const {
        return m_tag;
    }
    template<typename Visitor>
    auto accept_template(Visitor&& visitor) {
        switch (m_tag) {
        case tag_t::rust:
            return visitor(::taihe::static_tag<tag_t::rust>, m_data.rust);
        case tag_t::cpp:
            return visitor(::taihe::static_tag<tag_t::cpp>, m_data.cpp);
        }
    }
    template<typename Visitor>
    auto accept_template(Visitor&& visitor) const {
        switch (m_tag) {
        case tag_t::rust:
            return visitor(::taihe::static_tag<tag_t::rust>, m_data.rust);
        case tag_t::cpp:
            return visitor(::taihe::static_tag<tag_t::cpp>, m_data.cpp);
        }
    }
    template<typename... Args>
    static CppOrRustBook make_rust(Args&&... args) {
        return make<tag_t::rust>(::std::forward<Args>(args)...);
    }
    template<typename... Args>
    CppOrRustBook const& emplace_rust(Args&&... args) {
        return emplace<tag_t::rust>(::std::forward<Args>(args)...);
    }
    bool holds_rust() const {
        return holds<tag_t::rust>();
    }
    auto* get_rust_ptr() {
        return get_ptr<tag_t::rust>();
    }
    auto const* get_rust_ptr() const {
        return get_ptr<tag_t::rust>();
    }
    auto& get_rust_ref() {
        return get_ref<tag_t::rust>();
    }
    auto const& get_rust_ref() const {
        return get_ref<tag_t::rust>();
    }
    template<typename... Args>
    static CppOrRustBook make_cpp(Args&&... args) {
        return make<tag_t::cpp>(::std::forward<Args>(args)...);
    }
    template<typename... Args>
    CppOrRustBook const& emplace_cpp(Args&&... args) {
        return emplace<tag_t::cpp>(::std::forward<Args>(args)...);
    }
    bool holds_cpp() const {
        return holds<tag_t::cpp>();
    }
    auto* get_cpp_ptr() {
        return get_ptr<tag_t::cpp>();
    }
    auto const* get_cpp_ptr() const {
        return get_ptr<tag_t::cpp>();
    }
    auto& get_cpp_ref() {
        return get_ref<tag_t::cpp>();
    }
    auto const& get_cpp_ref() const {
        return get_ref<tag_t::cpp>();
    }
    template<typename Visitor>
    auto accept(Visitor&& visitor) {
        switch (m_tag) {
        case tag_t::rust:
            return visitor.rust(m_data.rust);
        case tag_t::cpp:
            return visitor.cpp(m_data.cpp);
        }
    }
    template<typename Visitor>
    auto accept(Visitor&& visitor) const {
        switch (m_tag) {
        case tag_t::rust:
            return visitor.rust(m_data.rust);
        case tag_t::cpp:
            return visitor.cpp(m_data.cpp);
        }
    }
private:
    tag_t m_tag;
    storage_t m_data;
};
}
namespace taihe {
inline bool same_impl(adl_helper_t, ::ohos::book::store::CppOrRustBook const& lhs, ::ohos::book::store::CppOrRustBook const& rhs) {
    return false || (lhs.holds_rust() && rhs.holds_rust() && same(lhs.get_rust_ref(), rhs.get_rust_ref())) || (lhs.holds_cpp() && rhs.holds_cpp() && same(lhs.get_cpp_ref(), rhs.get_cpp_ref()));
}
}
namespace taihe {
inline auto hash_impl(adl_helper_t, ::ohos::book::store::CppOrRustBook const& val) -> ::std::size_t {
    switch (val.get_tag()) {
        ::std::size_t seed;
    case ::ohos::book::store::CppOrRustBook::tag_t::rust:
        seed = (::std::size_t)::ohos::book::store::CppOrRustBook::tag_t::rust;
        return seed ^ (0x9e3779b9 + (seed << 6) + (seed >> 2) + hash(val.get_rust_ref()));
    case ::ohos::book::store::CppOrRustBook::tag_t::cpp:
        seed = (::std::size_t)::ohos::book::store::CppOrRustBook::tag_t::cpp;
        return seed ^ (0x9e3779b9 + (seed << 6) + (seed >> 2) + hash(val.get_cpp_ref()));
    }
}
}
namespace taihe {
template<>
struct as_abi<::ohos::book::store::CppOrRustBook> {
    using type = struct ohos_book_store_CppOrRustBook_t;
};
template<>
struct as_abi<::ohos::book::store::CppOrRustBook const&> {
    using type = struct ohos_book_store_CppOrRustBook_t const*;
};
template<>
struct as_param<::ohos::book::store::CppOrRustBook> {
    using type = ::ohos::book::store::CppOrRustBook const&;
};
}
