#pragma once
#include "taihe/common.hpp"
#include "ohos.book.store.MapOption.proj.0.hpp"
#include "ohos.book.store.MapOption.abi.1.h"
#include "ohos.book.store.Book.proj.1.hpp"
#include "taihe/array.hpp"
namespace ohos::book::store {
struct MapOption {
    enum class tag_t : int {
        one_book,
        many_books,
    };
    union storage_t {
        storage_t() {}
        ~storage_t() {}
        ::ohos::book::store::Book one_book;
        ::taihe::array<::ohos::book::store::Book> many_books;
    };
    MapOption(MapOption const& other) : m_tag(other.m_tag) {
        switch (m_tag) {
        case tag_t::one_book:
            new (&m_data.one_book) decltype(m_data.one_book)(other.m_data.one_book);
            break;
        case tag_t::many_books:
            new (&m_data.many_books) decltype(m_data.many_books)(other.m_data.many_books);
            break;
        default:
            break;
        }
    }
    MapOption(MapOption&& other) : m_tag(other.m_tag) {
        switch (m_tag) {
        case tag_t::one_book:
            new (&m_data.one_book) decltype(m_data.one_book)(::std::move(other.m_data.one_book));
            break;
        case tag_t::many_books:
            new (&m_data.many_books) decltype(m_data.many_books)(::std::move(other.m_data.many_books));
            break;
        default:
            break;
        }
    }
    MapOption& operator=(MapOption const& other) {
        if (this != &other) {
            ::std::destroy_at(this);
            new (this) MapOption(other);
        }
        return *this;
    }
    MapOption& operator=(MapOption&& other) {
        if (this != &other) {
            ::std::destroy_at(this);
            new (this) MapOption(::std::move(other));
        }
        return *this;
    }
    ~MapOption() {
        switch (m_tag) {
        case tag_t::one_book:
            ::std::destroy_at(&m_data.one_book);
            break;
        case tag_t::many_books:
            ::std::destroy_at(&m_data.many_books);
            break;
        default:
            break;
        }
    }
    template<typename... Args>
    MapOption(::taihe::static_tag_t<tag_t::one_book>, Args&&... args) : m_tag(tag_t::one_book) {
        new (&m_data.one_book) decltype(m_data.one_book)(::std::forward<Args>(args)...);
    }
    template<typename... Args>
    MapOption(::taihe::static_tag_t<tag_t::many_books>, Args&&... args) : m_tag(tag_t::many_books) {
        new (&m_data.many_books) decltype(m_data.many_books)(::std::forward<Args>(args)...);
    }
    template<tag_t tag, typename... Args>
    static MapOption make(Args&&... args) {
        return MapOption(::taihe::static_tag<tag>, ::std::forward<Args>(args)...);
    }
    template<tag_t tag, typename... Args>
    MapOption const& emplace(Args&&... args) {
        ::std::destroy_at(this);
        new (this) MapOption(::taihe::static_tag<tag>, ::std::forward<Args>(args)...);
        return *this;
    }
    template<tag_t tag>
    auto& get_ref() {
        if constexpr (tag == tag_t::one_book) {
            return m_data.one_book;
        }
        if constexpr (tag == tag_t::many_books) {
            return m_data.many_books;
        }
    }
    template<tag_t tag>
    auto* get_ptr() {
        return m_tag == tag ? &get_ref<tag>() : nullptr;
    }
    template<tag_t tag>
    auto const& get_ref() const {
        if constexpr (tag == tag_t::one_book) {
            return m_data.one_book;
        }
        if constexpr (tag == tag_t::many_books) {
            return m_data.many_books;
        }
    }
    template<tag_t tag>
    auto const* get_ptr() const {
        return m_tag == tag ? &get_ref<tag>() : nullptr;
    }
    template<tag_t tag>
    bool holds() const {
        return m_tag == tag;
    }
    tag_t get_tag() const {
        return m_tag;
    }
    template<typename Visitor>
    auto accept_template(Visitor&& visitor) {
        switch (m_tag) {
        case tag_t::one_book:
            return visitor(::taihe::static_tag<tag_t::one_book>, m_data.one_book);
        case tag_t::many_books:
            return visitor(::taihe::static_tag<tag_t::many_books>, m_data.many_books);
        }
    }
    template<typename Visitor>
    auto accept_template(Visitor&& visitor) const {
        switch (m_tag) {
        case tag_t::one_book:
            return visitor(::taihe::static_tag<tag_t::one_book>, m_data.one_book);
        case tag_t::many_books:
            return visitor(::taihe::static_tag<tag_t::many_books>, m_data.many_books);
        }
    }
    template<typename... Args>
    static MapOption make_one_book(Args&&... args) {
        return make<tag_t::one_book>(::std::forward<Args>(args)...);
    }
    template<typename... Args>
    MapOption const& emplace_one_book(Args&&... args) {
        return emplace<tag_t::one_book>(::std::forward<Args>(args)...);
    }
    bool holds_one_book() const {
        return holds<tag_t::one_book>();
    }
    auto* get_one_book_ptr() {
        return get_ptr<tag_t::one_book>();
    }
    auto const* get_one_book_ptr() const {
        return get_ptr<tag_t::one_book>();
    }
    auto& get_one_book_ref() {
        return get_ref<tag_t::one_book>();
    }
    auto const& get_one_book_ref() const {
        return get_ref<tag_t::one_book>();
    }
    template<typename... Args>
    static MapOption make_many_books(Args&&... args) {
        return make<tag_t::many_books>(::std::forward<Args>(args)...);
    }
    template<typename... Args>
    MapOption const& emplace_many_books(Args&&... args) {
        return emplace<tag_t::many_books>(::std::forward<Args>(args)...);
    }
    bool holds_many_books() const {
        return holds<tag_t::many_books>();
    }
    auto* get_many_books_ptr() {
        return get_ptr<tag_t::many_books>();
    }
    auto const* get_many_books_ptr() const {
        return get_ptr<tag_t::many_books>();
    }
    auto& get_many_books_ref() {
        return get_ref<tag_t::many_books>();
    }
    auto const& get_many_books_ref() const {
        return get_ref<tag_t::many_books>();
    }
    template<typename Visitor>
    auto accept(Visitor&& visitor) {
        switch (m_tag) {
        case tag_t::one_book:
            return visitor.one_book(m_data.one_book);
        case tag_t::many_books:
            return visitor.many_books(m_data.many_books);
        }
    }
    template<typename Visitor>
    auto accept(Visitor&& visitor) const {
        switch (m_tag) {
        case tag_t::one_book:
            return visitor.one_book(m_data.one_book);
        case tag_t::many_books:
            return visitor.many_books(m_data.many_books);
        }
    }
private:
    tag_t m_tag;
    storage_t m_data;
};
}
namespace taihe {
inline bool same_impl(adl_helper_t, ::ohos::book::store::MapOption const& lhs, ::ohos::book::store::MapOption const& rhs) {
    return false || (lhs.holds_one_book() && rhs.holds_one_book() && same(lhs.get_one_book_ref(), rhs.get_one_book_ref())) || (lhs.holds_many_books() && rhs.holds_many_books() && same(lhs.get_many_books_ref(), rhs.get_many_books_ref()));
}
}
namespace taihe {
inline auto hash_impl(adl_helper_t, ::ohos::book::store::MapOption const& val) -> ::std::size_t {
    switch (val.get_tag()) {
        ::std::size_t seed;
    case ::ohos::book::store::MapOption::tag_t::one_book:
        seed = (::std::size_t)::ohos::book::store::MapOption::tag_t::one_book;
        return seed ^ (0x9e3779b9 + (seed << 6) + (seed >> 2) + hash(val.get_one_book_ref()));
    case ::ohos::book::store::MapOption::tag_t::many_books:
        seed = (::std::size_t)::ohos::book::store::MapOption::tag_t::many_books;
        return seed ^ (0x9e3779b9 + (seed << 6) + (seed >> 2) + hash(val.get_many_books_ref()));
    }
}
}
namespace taihe {
template<>
struct as_abi<::ohos::book::store::MapOption> {
    using type = struct ohos_book_store_MapOption_t;
};
template<>
struct as_abi<::ohos::book::store::MapOption const&> {
    using type = struct ohos_book_store_MapOption_t const*;
};
template<>
struct as_param<::ohos::book::store::MapOption> {
    using type = ::ohos::book::store::MapOption const&;
};
}
